@@ -1,25 +1,21 @@
+#include <volk/volk.h>
 #include "qa_utils.h"
 
-#include <boost/foreach.hpp>
-#include <boost/tokenizer.hpp>
-#include <boost/lexical_cast.hpp>
-#include <boost/typeof/typeof.hpp>
-#include <boost/type_traits.hpp>
+#include <volk/volk.h>                              // for volk_func_desc_t
+#include <volk/volk_malloc.h>                       // for volk_free, volk_m...
 
-#include <iostream>
-#include <cstring>
-#include <fstream>
-#include <vector>
-#include <map>
-#include <list>
-#include <ctime>
-#include <cmath>
-#include <limits>
-
-#include <volk/volk.h>
-#include <volk/volk_cpu.h>
-#include <volk/volk_common.h>
-#include <volk/volk_malloc.h>
+#include <assert.h>                                 // for assert
+#include <stdint.h>                                 // for uint16_t, uint64_t
+#include <sys/time.h>                               // for CLOCKS_PER_SEC
+#include <sys/types.h>                              // for int16_t, int32_t
+#include <cmath>                                    // for sqrt, fabs, abs
+#include <cstring>                                  // for memcpy, memset
+#include <ctime>                                    // for clock
+#include <fstream>                                  // for operator<<, basic...
+#include <iostream>                                 // for cout, cerr
+#include <limits>                                   // for numeric_limits
+#include <map>                                      // for map, map<>::mappe...
+#include <vector>                                   // for vector, _Bit_refe...
 
 float uniform() {
   return 2.0f * ((float) rand() / RAND_MAX - 0.5f);	// uniformly (-1, 1)
@@ -53,8 +49,8 @@ void load_random_data(void *data, volk_type_t type, unsigned int n) {
                 else ((uint32_t *)data)[i] = (uint32_t) scaled_rand;
             break;
             case 2:
-                if(type.is_signed) ((int16_t *)data)[i] = (int16_t) scaled_rand;
-                else ((uint16_t *)data)[i] = (uint16_t) scaled_rand;
+                if(type.is_signed) ((int16_t *)data)[i] = (int16_t)((int16_t) scaled_rand % 8);
+                else ((uint16_t *)data)[i] = (uint16_t) ((int16_t) scaled_rand % 8);
             break;
             case 1:
                 if(type.is_signed) ((int8_t *)data)[i] = (int8_t) scaled_rand;
@@ -77,6 +73,22 @@ static std::vector<std::string> get_arch_list(volk_func_desc_t desc) {
     return archlist;
 }
 
+template <typename T>
+T volk_lexical_cast(const std::string& str)
+{
+    for (unsigned int c_index = 0; c_index < str.size(); ++c_index) {
+        if (str.at(c_index) < '0' || str.at(c_index) > '9') {
+            throw "not all numbers!";
+        }
+    }
+    T var;
+    std::istringstream iss;
+    iss.str(str);
+    iss >> var;
+    // deal with any error bits that may have been set on the stream
+    return var;
+}
+
 volk_type_t volk_type_from_string(std::string name) {
     volk_type_t type;
     type.is_float = false;
@@ -102,8 +114,8 @@ volk_type_t volk_type_from_string(std::string name) {
         throw std::string("no size spec in type ").append(name);
     }
     //will throw if malformed
-    int size = boost::lexical_cast<int>(name.substr(0, last_size_pos+1));
-
+    int size = volk_lexical_cast<int>(name.substr(0, last_size_pos+1));
+    
     assert(((size % 8) == 0) && (size <= 64) && (size != 0));
     type.size = size/8; //in bytes
 
@@ -129,14 +141,28 @@ volk_type_t volk_type_from_string(std::string name) {
     return type;
 }
 
+std::vector<std::string> split_signature(const std::string &protokernel_signature) {
+    std::vector<std::string> signature_tokens;
+    std::string token;
+    for (unsigned int loc = 0; loc < protokernel_signature.size(); ++loc) {
+        if (protokernel_signature.at(loc) == '_') {
+            // this is a break
+            signature_tokens.push_back(token);
+            token = "";
+        } else {
+            token.push_back(protokernel_signature.at(loc));
+        }
+    }
+    // Get the last one to the end of the string
+    signature_tokens.push_back(token);
+    return signature_tokens;
+}
+
 static void get_signatures_from_name(std::vector<volk_type_t> &inputsig,
                                    std::vector<volk_type_t> &outputsig,
                                    std::string name) {
-    boost::char_separator<char> sep("_");
-    boost::tokenizer<boost::char_separator<char> > tok(name, sep);
-    std::vector<std::string> toked;
-    tok.assign(name);
-    toked.assign(tok.begin(), tok.end());
+
+    std::vector<std::string> toked = split_signature(name);
 
     assert(toked[0] == "volk");
     toked.erase(toked.begin());
@@ -147,7 +173,8 @@ static void get_signatures_from_name(std::vector<volk_type_t> &inputsig,
     enum { SIDE_INPUT, SIDE_NAME, SIDE_OUTPUT } side = SIDE_INPUT;
     std::string fn_name;
     volk_type_t type;
-    BOOST_FOREACH(std::string token, toked) {
+    for (unsigned int token_index = 0; token_index < toked.size(); ++token_index) {
+        std::string token = toked[token_index];
         try {
             type = volk_type_from_string(token);
             if(side == SIDE_NAME) side = SIDE_OUTPUT; //if this is the first one after the name...
@@ -155,10 +182,10 @@ static void get_signatures_from_name(std::vector<volk_type_t> &inputsig,
             if(side == SIDE_INPUT) inputsig.push_back(type);
             else outputsig.push_back(type);
         } catch (...){
-            if(token[0] == 'x') { //it's a multiplier
+            if(token[0] == 'x' && (token.size() > 1) && (token[1] > '0' || token[1] < '9')) { //it's a multiplier
                 if(side == SIDE_INPUT) assert(inputsig.size() > 0);
                 else assert(outputsig.size() > 0);
-                int multiplier = boost::lexical_cast<int>(token.substr(1, token.size()-1)); //will throw if invalid
+                int multiplier = volk_lexical_cast<int>(token.substr(1, token.size()-1)); //will throw if invalid
                 for(int i=1; i<multiplier; i++) {
                     if(side == SIDE_INPUT) inputsig.push_back(inputsig.back());
                     else outputsig.push_back(outputsig.back());
@@ -231,7 +258,8 @@ bool fcompare(t *in1, t *in2, unsigned int vlen, float tol) {
             {
                 fail=true;
                 if(print_max_errs-- > 0) {
-                    std::cout << "offset " << i << " in1: " << t(((t *)(in1))[i]) << " in2: " << t(((t *)(in2))[i]) << std::endl;
+                    std::cout << "offset " << i << " in1: " << t(((t *)(in1))[i]) << " in2: " << t(((t *)(in2))[i]);
+                    std::cout << " tolerance was: " << tol << std::endl;
                 }
             }
         }
@@ -239,7 +267,8 @@ bool fcompare(t *in1, t *in2, unsigned int vlen, float tol) {
         else if(fabs(((t *)(in1))[i] - ((t *)(in2))[i])/fabs(((t *)in1)[i]) > tol) {
             fail=true;
             if(print_max_errs-- > 0) {
-                std::cout << "offset " << i << " in1: " << t(((t *)(in1))[i]) << " in2: " << t(((t *)(in2))[i]) << std::endl;
+                std::cout << "offset " << i << " in1: " << t(((t *)(in1))[i]) << " in2: " << t(((t *)(in2))[i]);
+                std::cout << " tolerance was: " << tol << std::endl;
             }
         }
     }
@@ -263,7 +292,8 @@ bool ccompare(t *in1, t *in2, unsigned int vlen, float tol) {
             {
                 fail=true;
                 if(print_max_errs-- > 0) {
-                    std::cout << "offset " << i/2 << " in1: " << in1[i] << " + " << in1[i+1] << "j  in2: " << in2[i] << " + " << in2[i+1] << "j" << std::endl;
+                    std::cout << "offset " << i/2 << " in1: " << in1[i] << " + " << in1[i+1] << "j  in2: " << in2[i] << " + " << in2[i+1] << "j";
+                    std::cout << " tolerance was: " << tol << std::endl;
                 }
             }
         }
@@ -271,7 +301,8 @@ bool ccompare(t *in1, t *in2, unsigned int vlen, float tol) {
         else if((err / norm) > tol) {
             fail=true;
             if(print_max_errs-- > 0) {
-                std::cout << "offset " << i/2 << " in1: " << in1[i] << " + " << in1[i+1] << "j  in2: " << in2[i] << " + " << in2[i+1] << "j" << std::endl;
+                std::cout << "offset " << i/2 << " in1: " << in1[i] << " + " << in1[i+1] << "j  in2: " << in2[i] << " + " << in2[i+1] << "j";
+                std::cout << " tolerance was: " << tol << std::endl;
             }
         }
     }
@@ -287,7 +318,8 @@ bool icompare(t *in1, t *in2, unsigned int vlen, unsigned int tol) {
       if(((unsigned int)abs(int(((t *)(in1))[i]) - int(((t *)(in2))[i]))) > tol) {
             fail=true;
             if(print_max_errs-- > 0) {
-                std::cout << "offset " << i << " in1: " << static_cast<int>(t(((t *)(in1))[i])) << " in2: " << static_cast<int>(t(((t *)(in2))[i])) << std::endl;
+                std::cout << "offset " << i << " in1: " << static_cast<int>(t(((t *)(in1))[i])) << " in2: " << static_cast<int>(t(((t *)(in2))[i]));
+                std::cout << " tolerance was: " << tol << std::endl;
             }
         }
     }
@@ -368,7 +400,7 @@ bool run_volk_tests(volk_func_desc_t desc,
     try {
         get_signatures_from_name(inputsig, outputsig, name);
     }
-    catch (boost::bad_lexical_cast& error) {
+    catch (std::exception &error) {
         std::cerr << "Error: unable to get function signature from kernel name" << std::endl;
         std::cerr << "  - " << name << std::endl;
         return false;
@@ -384,7 +416,8 @@ bool run_volk_tests(volk_func_desc_t desc,
         }
     }
     std::vector<void *> inbuffs;
-    BOOST_FOREACH(volk_type_t sig, inputsig) {
+    for (unsigned int inputsig_index = 0; inputsig_index < inputsig.size(); ++ inputsig_index) {
+        volk_type_t sig = inputsig[inputsig_index];
         if(!sig.is_scalar) //we don't make buffers for scalars
           inbuffs.push_back(mem_pool.get_new(vlen*sig.size*(sig.is_complex ? 2 : 1)));
     }
@@ -516,10 +549,21 @@ bool run_volk_tests(volk_func_desc_t desc,
                         }
                         break;
                     case 4:
-                        if(both_sigs[j].is_signed) {
-                            fail = icompare((int32_t *) test_data[generic_offset][j], (int32_t *) test_data[i][j], vlen*(both_sigs[j].is_complex ? 2 : 1), tol_i);
-                        } else {
-                            fail = icompare((uint32_t *) test_data[generic_offset][j], (uint32_t *) test_data[i][j], vlen*(both_sigs[j].is_complex ? 2 : 1), tol_i);
+                        if(both_sigs[j].is_complex) {
+                            if(both_sigs[j].is_signed) {
+                                fail = icompare((int16_t *) test_data[generic_offset][j], (int16_t *) test_data[i][j], vlen*(both_sigs[j].is_complex ? 2 : 1), tol_i);
+                            } else {
+                                fail = icompare((uint16_t *) test_data[generic_offset][j], (uint16_t *) test_data[i][j], vlen*(both_sigs[j].is_complex ? 2 : 1), tol_i);
+                            }
+                        }
+                        else {
+                            if (both_sigs[j].is_signed) {
+                                fail = icompare((int32_t *) test_data[generic_offset][j], (int32_t *) test_data[i][j],
+                                                vlen * (both_sigs[j].is_complex ? 2 : 1), tol_i);
+                            } else {
+                                fail = icompare((uint32_t *) test_data[generic_offset][j], (uint32_t *) test_data[i][j],
+                                                vlen * (both_sigs[j].is_complex ? 2 : 1), tol_i);
+                            }
                         }
                         break;
                     case 2:
@@ -571,15 +615,14 @@ bool run_volk_tests(volk_func_desc_t desc,
 
     std::cout << "Best aligned arch: " << best_arch_a << std::endl;
     std::cout << "Best unaligned arch: " << best_arch_u << std::endl;
-    if(results) {
-        if(puppet_master_name == "NULL") {
-            results->back().config_name = name;
-        } else {
-            results->back().config_name = puppet_master_name;
-        }
-        results->back().best_arch_a = best_arch_a;
-        results->back().best_arch_u = best_arch_u;
+
+    if(puppet_master_name == "NULL") {
+        results->back().config_name = name;
+    } else {
+        results->back().config_name = puppet_master_name;
     }
+    results->back().best_arch_a = best_arch_a;
+    results->back().best_arch_u = best_arch_u;
 
     return fail_global;
 }
